@@ -1,13 +1,16 @@
 package Bryar;
+
 use Bryar::Config;
 use Time::Local;
 use Bryar::Comment;
 use Calendar::Simple;
+use DateTime;
+
 use 5.006;
 use strict;
 use warnings;
 use Carp;
-our $VERSION = '2.6';
+our $VERSION = '3.0';
 
 =head1 NAME
 
@@ -36,9 +39,7 @@ web server. Then do what it tells you.
 The long answer:
 
 The only front-end working in this release is the CGI one; please don't
-try this in mod_perl yet. C<Bryar::Frontend::Mod_perl> is experimental,
-and while I'm running it on my live server, it doesn't actually offer
-any advantages yet.
+try this in mod_perl yet.
 
 You'll need to write a little driver script which sets some parameters.
 For instance, my F<bryar.cgi> looks like this:
@@ -46,9 +47,9 @@ For instance, my F<bryar.cgi> looks like this:
     #!/usr/bin/perl
     use Bryar;
     Bryar->go(
-         name => "Themes, Dreams and Crazy Schemes",
-         description => "Simon Cozens' weblog",
-         baseurl => "http://blog.simon-cozens.org/bryar.cgi"
+	 name => "Themes, Dreams and Crazy Schemes",
+	 description => "Simon Cozens' weblog",
+	 baseurl => "http://blog.simon-cozens.org/bryar.cgi"
     );
 
 You can get away without any configuration options, but it's probably
@@ -67,6 +68,7 @@ full of other options. I could, for instance, get away with
     name: Themes, Dreams and Crazy Schemes
     description: Simon Cozens' weblog
     baseurl: http://blog.simon-cozens.org/bryar.cgi
+    email: something@example.com
 
 in a F<bryar.conf>, and then would be able to use C<< Bryar->go() >>
 with no further parameters.
@@ -157,11 +159,12 @@ you're programming your blog to do clever stuff. Use C<go> instead.
 
 =cut
 
-
 sub new {
     my $class = shift;
+	my %args = (configclass => 'Bryar::Config', @_);
+	eval "require $args{configclass}" or die $@;
     bless {
-        config => Bryar::Config->new(@_)
+	config => $args{configclass}->new(@_)
     }, $class;
 }
 
@@ -178,85 +181,94 @@ interested in setting, see C<Bryar::Config>.
 sub go {
     my $self = shift;
     if (not ref $self) {
-        # We need to acquire a Bryar object.
-        my %args = @_;
+	# We need to acquire a Bryar object.
+	my %args = @_;
 
-        # If we're running under mod_perl, then we want to check to see
-        # if a datadir or config file has been specified.
-        if (exists $ENV{GATEWAY_INTERFACE} and 
-            $ENV{'GATEWAY_INTERFACE'} =~ /^CGI-Perl\//) {
-            require Apache;
-            my $r = Apache->request;
-            my $x;
-            $args{datadir} = $x if $x = $r->dir_config("BryarDatadir");
-            $args{config}  = $x if $x = $r->dir_config("BryarConfig");
-        }
-        $self = Bryar->new(%args);
+	# If we're running under mod_perl, then we want to check to see
+	# if a datadir or config file has been specified.
+	if (exists $ENV{GATEWAY_INTERFACE} and 
+	    $ENV{'GATEWAY_INTERFACE'} =~ /^CGI-Perl\//) {
+	    require Apache;
+	    my $r = Apache->request;
+	    my $x;
+	    $args{datadir} = $x if $x = $r->dir_config("BryarDatadir");
+	    $args{config}  = $x if $x = $r->dir_config("BryarConfig");
+	}
+	$self = Bryar->new(%args);
     }
-    my $frontend = $self->{config}->frontend();
-    $frontend->init($self) if $frontend->can("init");
-
-    $self->_doit;
+    my $frontend = $self->config->frontend();
+    $frontend->init($self->config) if $frontend->can("init");
+    
+	$self->_doit;
 }
 
 sub _doit {
     my $self = shift;
-    my %args = $self->{config}->frontend()->parse_args($self);
-    my @documents = $self->{config}->collector()->collect($self, %args);
+    my %args = $self->config->frontend()->parse_args($self->config, @_);
+	$self->{arguments} = \%args;
+
+    my @documents = $self->config->collector()->collect($self->config, %args);
 
     $args{format} ||= "html";
 
-    $self->{config}->frontend()->output(
-        $self->{config}->renderer->generate($args{format}, $self, @documents)
+    $self->config->frontend()->output(
+		$self->config->renderer->generate(
+					$self->{arguments}{format},
+					$self,
+					@documents
+				)
     );
 }
 
 =head2 posts_calendar
 
+TODO:  Move this out to something that is more flexible.
+
 Return a data structure containing the days and weeks of a given month and
 year with blog posts attached. See the C<calendar> template for an example.
 
 =cut
 
-my @mons = qw(x Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
-
 sub posts_calendar {
-        my ($self, $month, $year) = @_;
+	my ($self, $month, $year) = @_;
 
-        $month ||= (localtime)[4];
-        $year  ||= (localtime)[5];
+    my $today = DateTime->today( time_zone => $self->config->{time_zone} );
 
-        my $this_month = timegm(0, 0, 0, 1, $month, $year);
-        my @documents = $self->{config}->collector->collect($self,
-                since => $this_month
-        );
+	$month ||= $today->month(); 
+	$year  ||= $today->year(); 
 
-        $month++;
-        $year += 1900;
+	my $this_month = DateTime->new( month => $month, year => $year,  time_zone => $self->config->{time_zone} );
 
-        # make an hash with keys the days with a post
-        my %posts = map { (gmtime($_->{epoch}))[3] => $_->{id} } @documents;
+	my @documents = $self->config->collector->collect(
+		$self->config,
+		since => $this_month->epoch()
+	);
 
-        my @m = calendar($month, $year);
-        my @month;
-        foreach my $week (@m) {
-                my @weekdays;
-                foreach my $day (@$week) {
-                        my $d = { day => $day };
-                        if ($day and exists $posts{$day}) {
-                                $d->{idlink} = $posts{$day};
-                                $d->{link} = "$year/$mons[$month]/$day";
-                        }
-                        push(@weekdays, $d);
-                }
+	# make an hash with keys the days with a post
+	my %posts = map { DateTime->from_epoch( epoch => $_->{epoch},  time_zone => $self->config->{time_zone} )->day() => $_->{id} } @documents;
+use Data::Dumper;
+print STDERR Dumper(\%posts);
 
-                # mark the first day of the week, if it exists
-                $weekdays[0]{sunday} = 1 if defined $weekdays[0]{day};
+	my @m = calendar($month, $year);
+	my @month;
+	foreach my $week (@m) {
+		my @weekdays;
+		foreach my $day (@$week) {
+			my $d = { day => $day };
+			if ($day and exists $posts{$day}) {
+				$d->{idlink} = $posts{$day};
+				$d->{link} = "$year/" . $this_month->month_abbr() . "/$day";
+			}
+			push(@weekdays, $d);
+		}
 
-                push(@month, \@weekdays);
-        }
+		# mark the first day of the week, if it exists
+		$weekdays[0]{sunday} = 1 if defined $weekdays[0]{day};
 
-        return @month;
+		push(@month, \@weekdays);
+	}
+
+	return { year => $year, month => $month, monthname => $this_month->month_name(), calendar => \@month };
 }
 
 =head2 config
@@ -283,6 +295,9 @@ Cache-Control support, as well as the calendar.
 
 Copyright (C) 2003, Simon Cozens C<simon@cpan.org>
 
+some parts Copyright 2007 David Cantrell C<david@cantrell.org.uk>
+
+
 =cut
 
 1;
